#################################################################################
#                                                                               #
#   2023 Giacomo Riccardo Hvaring                                               #
#                                                                               #
#   Lybrary of functions for experiments with the Ettus Research SDR.           #
#                                                                               #
#################################################################################

import numpy as np
import uhd
import time


single_tone = lambda x, f, phase: np.exp(2j * np.pi * (f * x + phase), dtype=np.complex64) # must be 64


def initialize_sdr(tx_frequency, sampling_rate):
    # find the sdr
    usrp = uhd.usrp.MultiUSRP()
    usrp.set_tx_freq(tx_frequency)
    usrp.set_tx_rate(sampling_rate)
    return usrp
    

def get_tx_metadata():
    return uhd.libpyuhd.types.tx_metadata()


def get_streamer(usrp, channels=None):
    stream_args = uhd.libpyuhd.usrp.stream_args("fc32", "sc8")
    stream_args.channels = [0] if channels is None else channels
    #stream_args.args = 'peak=1'
    return usrp.get_tx_stream(stream_args)


def fill_buffer(waveform, buffer_size):
    repetitions = buffer_size // waveform.shape[-1]
    tiled = waveform
    if repetitions > 0:
        tiled = np.tile(waveform, repetitions)
    return tiled


def send(waveform, total_points_to_send, streamer):
    '''Send waveform in loop until total_points_to_send are sent.'''
    sample = waveform
    metadata = get_tx_metadata()
    metadata.start_of_burst = True
    sent = 0

    #metadata.has_time_spec = False

    while sent < total_points_to_send:
        remaining = total_points_to_send - sent

        if remaining < sample.size:
            sample = sample[:remaining]

        sent += streamer.send(sample, metadata)
        metadata.start_of_burst = False
    
    metadata.end_of_burst = True  # terminate Tx
    streamer.send(np.zeros(1), metadata)

    return sent
